@@ -1,5 +1,5 @@
 /* asn1Decoding.c ---  program to generate an ASN1 type from a DER coding.
- * Copyright (C) 2002-2014 Free Software Foundation, Inc.
+ * Copyright (C) 2002-2021 Free Software Foundation, Inc.
  *
  * This file is part of LIBTASN1.
  *
@@ -25,14 +25,15 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <getopt.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 
 #include <libtasn1.h>
 
-#include <progname.h>
-#include <version-etc.h>
-#include <read-file.h>
 #include "benchmark.h"
 
+#define program_name "asn1Decoding"
+
 static int decode (asn1_node definitions, const char *typeName, void *der,
 		   int der_len, int benchmark, int strict);
 
@@ -59,19 +60,61 @@ described in ASN.1 DEFINITIONS file, and print decoded structures.\n\
       printf ("\
   -b, --benchmark       perform a benchmark on decoding\n\
   -s, --strict          use strict DER decoding\n\
+  -t, --no-time-strict  use strict DER decoding but not in time fields\n\
   -h, --help            display this help and exit\n\
   -v, --version         output version information and exit\n");
-      emit_bug_reporting_address ();
+     printf ("Report bugs to "PACKAGE_BUGREPORT);
     }
   exit (status);
 }
 
+static char *read_binary_file(const char *file, size_t *l)
+{
+  FILE *fp;
+  struct stat st;
+  char *out;
+
+  if (stat(file, &st) == -1)
+    {
+      fprintf(stderr, "Error reading file size!\n");
+      exit(1);
+    }
+
+  fp = fopen(file, "rb");
+  if (fp == NULL)
+    {
+      fprintf(stderr, "Error reading file!\n");
+      exit(1);
+    }
+
+  out = malloc(st.st_size+1);
+  if (out == NULL)
+    {
+      fprintf(stderr, "Memory error!\n");
+      exit(1);
+    }
+
+  *l = fread(out, 1, st.st_size, fp);
+  if ((off_t)*l != st.st_size)
+    {
+      fprintf(stderr, "Error reading contents (got: %ld, expected %ld)!\n",
+              (long)*l, (long)st.st_size);
+      exit(1);
+    }
+
+  out[*l] = 0;
+
+  fclose(fp);
+  return out;
+}
+
 int
 main (int argc, char *argv[])
 {
   static const struct option long_options[] = {
     {"help", no_argument, 0, 'h'},
     {"strict", no_argument, 0, 's'},
+    {"no-time-strict", no_argument, 0, 't'},
     {"debug", no_argument, 0, 'd'},
     {"benchmark", no_argument, 0, 'b'},
     {"version", no_argument, 0, 'v'},
@@ -87,18 +130,16 @@ main (int argc, char *argv[])
   int asn1_result = ASN1_SUCCESS;
   unsigned char *der;
   int der_len = 0, benchmark = 0;
-  int strict = 0, debug = 0;
+  int flags = 0, debug = 0;
   /* FILE *outputFile; */
 
-  set_program_name (argv[0]);
-
   opterr = 0;			/* disable error messages from getopt */
 
   while (1)
     {
 
       option_result =
-	getopt_long (argc, argv, "hbdsvc", long_options, &option_index);
+	getopt_long (argc, argv, "hbdsvtc", long_options, &option_index);
 
       if (option_result == -1)
 	break;
@@ -115,11 +156,15 @@ main (int argc, char *argv[])
 	  debug = 1;
 	  break;
 	case 's':
-	  strict = 1;
+	case 't':
+	  flags |= ASN1_DECODE_FLAG_STRICT_DER;
+	  if (option_result == 't')
+	    flags |= ASN1_DECODE_FLAG_ALLOW_INCORRECT_TIME;
 	  break;
 	case 'v':		/* VERSION */
-	  version_etc (stdout, program_name, PACKAGE, VERSION,
-		       "Fabio Fiorina", NULL);
+	  printf(program_name" "PACKAGE" " VERSION"\n");
+	  printf("Copyright (C) 2017-2021 Free Software Foundation, Inc.\n\n");
+	  printf("Written by Fabio Fiorina\n");
 	  exit (0);
 	  break;
 	case '?':		/* UNKNOW OPTION */
@@ -214,7 +259,8 @@ main (int argc, char *argv[])
   /*
      der_len=0;
      outputFile=fopen("data.p12","w");
-     while(fscanf(inputFile,"%c",der+der_len) != EOF){
+     while(fscanf(inputFile,"%c",der+der_len) != EOF)
+     {
      if((der_len>=0x11) && (der_len<=(0xe70)))
      fprintf(outputFile,"%c",der[der_len]);
      der_len++;
@@ -223,7 +269,7 @@ main (int argc, char *argv[])
      fclose(inputFile);
    */
 
-  if (decode (definitions, typeName, der, der_len, benchmark, strict) != ASN1_SUCCESS)
+  if (decode (definitions, typeName, der, der_len, benchmark, flags) != ASN1_SUCCESS)
     {
       asn1_delete_structure (&definitions);
       free (inputFileAsnName);
@@ -249,7 +295,7 @@ main (int argc, char *argv[])
 
 static int
 simple_decode (asn1_node definitions, const char *typeName, void *der,
-	       int der_len, int benchmark, int strict)
+	       int der_len, int benchmark, int flags)
 {
 
   int asn1_result;
@@ -269,9 +315,9 @@ simple_decode (asn1_node definitions, const char *typeName, void *der,
       return asn1_result;
     }
 
-  if (strict != 0)
+  if (flags != 0)
     asn1_result =
-      asn1_der_decoding2(&structure, der, &der_len, ASN1_DECODE_FLAG_STRICT_DER, errorDescription);
+      asn1_der_decoding2(&structure, der, &der_len, flags, errorDescription);
   else
     asn1_result =
       asn1_der_decoding (&structure, der, der_len, errorDescription);
@@ -297,19 +343,19 @@ simple_decode (asn1_node definitions, const char *typeName, void *der,
 
 static int
 decode (asn1_node definitions, const char *typeName, void *der, int der_len,
-	int benchmark, int strict)
+	int benchmark, int flags)
 {
   struct benchmark_st st;
 
   if (benchmark == 0)
-    return simple_decode (definitions, typeName, der, der_len, benchmark, strict);
+    return simple_decode (definitions, typeName, der, der_len, benchmark, flags);
   else
     {
       start_benchmark (&st);
 
       do
 	{
-	  simple_decode (definitions, typeName, der, der_len, benchmark, strict);
+	  simple_decode (definitions, typeName, der, der_len, benchmark, flags);
 	  st.size++;
 	}
       while (benchmark_must_finish == 0);
