@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2013 Free Software Foundation, Inc.
+ * Copyright (C) 2011-2021 Free Software Foundation, Inc.
  *
  * This file is part of GnuTLS.
  *
@@ -17,19 +17,27 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#ifndef BENCHMARK_H
+# define BENCHMARK_H
+
+#include <config.h>
 #include <sys/time.h>
 #include <time.h>
 #include <signal.h>
 #if defined _WIN32
 #include <windows.h>
 #endif
-#include "timespec.h"		/* gnulib gettime */
+#include <time.h>
 
 typedef void (*sighandler_t) (int);
 
 struct benchmark_st
 {
+#ifdef HAVE_CLOCK_GETTIME
   struct timespec start;
+#else
+  struct timeval start;
+#endif
   unsigned long size;
   sighandler_t old_handler;
 #if defined _WIN32
@@ -50,3 +58,5 @@ timespec_sub_ms (struct timespec *a, struct timespec *b)
   return (a->tv_sec * 1000 + a->tv_nsec / (1000 * 1000) -
 	  (b->tv_sec * 1000 + b->tv_nsec / (1000 * 1000)));
 }
+
+#endif /* BENCHMARK_H */
